function make_spirals(niters,nmult)
%run this from the main FMM3Dc directory

%get seed data from Simulink model output
%spirals=bodies.signals.values;
%load seed data from file
file=load('examples/galaxy_collide/spirals_t100.mat');
spirals=file.spirals_t100;
nsteps=size(spirals,3);

SolarMass = 1.9891e+30; % In kg
SpeedOfLight = 299792458; % in m/s
YearInSeconds = 365*24*60*60;
LightYear = SpeedOfLight*YearInSeconds;
Parsec = 3.26*LightYear;
rad = 18000*Parsec;
innerrad = rad/3;
dr=rad/20; % radius of spheroid on new stars around each existing star
a=1; % control size of spheroid
b=1;
c=0.1;
%niters=4;
%nmult=10;
nscale=0.67;

%no. of stars in each galaxy
nstars=size(spirals,1)/2;

m=spirals(:,1,nsteps);
x=spirals(:,2,nsteps);
y=spirals(:,3,nsteps);
z=spirals(:,4,nsteps);
vx=spirals(:,5,nsteps);
vy=spirals(:,6,nsteps);
vz=spirals(:,7,nsteps);

%sort 1 galaxy from the other by (x,y) coords, ignoring 2 black holes
[A,I] = sort(x(3:end));
% correct indices
I=I+2;
I1=I(1:nstars-1);
I2=I(nstars:2*(nstars-1));
% galaxy 1
x1=x(I1);
y1=y(I1);
z1=z(I1);
vx1=vx(I1);
vy1=vy(I1);
vz1=vz(I1);
m1=m(I1);
% galaxy 2
x2=x(I2);
y2=y(I2);
z2=z(I2);
vx2=vx(I2);
vy2=vy(I2);
vz2=vz(I2);
m2=m(I2);

% add an extra star to replace black hole
% and make nstars a round number
xyz = randn(1,3)*dr;
x1=[x1(end)+xyz(1,1);x1];
y1=[y1(end)+xyz(1,2);y1];
z1=[z1(end)+xyz(1,3);z1];
vx1=[vx1(end);vx1];
vy1=[vy1(end);vy1];
vz1=[vz1(end);vz1];
m1=[(rand*20+4)*SolarMass;m1];

% figure,plot3(x1,y1,z1,'w.')
% axis equal;
% hold
% plot3(x2,y2,z2,'r.');

iter=0;
oldnum=nstars;
newnum=oldnum;
clear x2 y2 z2 m2 vx2 vy2 vz2;

while iter<niters,

    % make clusters around existing stars
    ind=1;
    newnum=oldnum*nmult;
    xnew=zeros(newnum,1);
    ynew=zeros(newnum,1);
    znew=zeros(newnum,1);
    vxnew=zeros(newnum,1);
    vynew=zeros(newnum,1);
    vznew=zeros(newnum,1);
    mnew=zeros(newnum,1);
    for i=1:oldnum,
        xnew(ind)=x1(i);
        ynew(ind)=y1(i);
        znew(ind)=z1(i);
        vxnew(ind)=vx1(i);
        vynew(ind)=vy1(i);
        vznew(ind)=vz1(i);
        mnew(ind)=m1(i);
        ind=ind+1;
        % make spheroid of randomly distributed stars
        r = dr*sqrt(rand(nmult,1));
        theta = 2*pi*rand(nmult,1);
        phi = 2*pi*rand(nmult,1);
        x = a*rand*r.*cos(theta).*sin(phi);
        y = b*rand*r.*sin(theta).*cos(phi);
        z = c*rand*r.*cos(phi);
        for j=1:nmult-1,
            xnew(ind)=x(j)+x1(i);
            ynew(ind)=y(j)+y1(i);
            znew(ind)=z(j)+z1(i);
            % in the absence of a better idea,
            % set new star velocities to origin star velocity
            vxnew(ind)=vx1(i);
            vynew(ind)=vy1(i);
            vznew(ind)=vz1(i);
            mnew(ind)=(rand*20+4)*SolarMass;
            ind=ind+1;
        end
    end
    oldnum=newnum;
    x1=xnew;
    y1=ynew;
    z1=znew;
    vx1=vxnew;
    vy1=vynew;
    vz1=vznew;
    m1=mnew;
    
    iter=iter+1;
    % change size of cluster
     dr = dr*nscale;
end

% add black hole back in
%x1=[x(2);x1];
%y1=[y(2);y1];
%z1=[z(2);z1];
%vx1=[vx(2);vx1];
%vy1=[vy(2);vy1];
%vz1=[vz(2);vz1];
%m1=[m(2);m1];
%newnum=newnum+1;

%figure,plot3(x1,y1,z1,'b.')
%axis equal
%whitebg('k');

% save to files
clear vx1 vy1 vz1 vxnew vynew vznew;
size(x1)
%z = [x1,y1,z1];
%v = [vx1,vy1,vz1];
m = m1;
clear x1 y1 z1 m1 xnew ynew znew mnew;
base='examples/galaxy_collide/';
%save(strcat(base,'spiralz_',int2str(newnum),'.mat'),'z','-v7.3');
save(strcat(base,'spiralm_',int2str(newnum),'.mat'),'m','-v7.3');
%save(strcat(base,'spiralvel_',int2str(newnum),'.mat'),'v','-v7.3');
