% this script generates some simple point-and-mass distributions and saves
% them to Matlab binary file.

% this ensures that the same random points are generated every time.
% the number is arbitrary.
rand('state',131345);
N=100000;
p =1;
dir='./experiments/paramsweep/scenarios/';

mkdir(dir);

% 1. random points in unit cube
base='random';
z = rand(N,3);
m = rand(N,1);

xx=z(:,1);
yy=z(:,2);
zz=z(:,3);
xyzm = cat(2,xx,yy,zz,m);

figure,scatter3(z(:,1),z(:,2),z(:,3))
xyzm= [z m]; 
mkdir(strcat(dir,base,int2str(N),'/'));
partition_mesh(xyzm,p,N,strcat(dir,base,int2str(N),'/'));
%save(strcat(dir,base,int2str(N),'z','.mat'),'z');
%save(strcat(dir,base,int2str(N),'m','.mat'),'m');
% uncomment to save z to generic uint8 format binary file: readable by FMM3D
% when compiled with -DNOMEX
%name=strcat(base,'z',int2str(N),'.dat')
%zfile=fopen(name,'w');
%fwrite(zfile,z,'double');

% 2. Gaussian dist in sphere
base='gaussian';
%mean = [0 0 0];
%std = 1;
%z = bsxfun(@plus,mean,std.*randn(N,3));
z = randn(N,3);
figure,scatter3(z(:,1),z(:,2),z(:,3))
m = rand(N,1);

xyzm= [z m]; 
mkdir(strcat(dir,base,int2str(N),'/'));
partition_mesh(xyzm,p,N,strcat(dir,base,int2str(N),'/'));
%save(strcat(dir,base,int2str(N),'z','.mat'),'z');
%save(strcat(dir,base,int2str(N),'m','.mat'),'m');

% 3. spherical shell
base='shell';
R=1;
s = 2*pi*rand(N,1);
t = 2*pi*rand(N,1);
xx = R*cos(s).*sin(t);
yy = R*sin(s).*sin(t);
zz = -R*cos(t);
z = cat(2,xx,yy,zz);
figure,scatter3(z(:,1),z(:,2),z(:,3))

xyzm= [z m]; 
mkdir(strcat(dir,base,int2str(N),'/'));
partition_mesh(xyzm,p,N,strcat(dir,base,int2str(N),'/'));
%save(strcat(dir,base,int2str(N),'z','.mat'),'z');
%save(strcat(dir,base,int2str(N),'m','.mat'),'m');

% 4. spiral
base='spiral';
size = 100;
particleMultiple = N/(size*10);
randOffset = 0.5;
A = 1:particleMultiple;
t = (0:1/size:10)*pi;
totSize= 10*size*particleMultiple;
%one to manny elements is created
t =  t(1:end-1);
%N = 10*particleMultiple*size;
X=[];
Y=[];
Z=[];

for h = t
    for i = A
       X = [X (sin(h) + rand()*randOffset-randOffset/2)*size + size *(randOffset/2 + 1)*pi/2];
       Y = [Y (cos(h) + rand()*randOffset-randOffset/2)*size + size *(randOffset/2 + 1)*pi/2];
       Z = [Z (h + rand()*randOffset*3-randOffset/2)*10];
    end
end

base='spiral';
m = rand(totSize,1);

figure
plot3(X,Y,Z,'o','markersize', 1)
z = [X;Y;Z];
z = transpose(z);
xyzm= [z m];
mkdir(strcat(dir,base,int2str(N),'/'));
partition_mesh(xyzm,p,N,strcat(dir,base,int2str(N),'/'));
%save(strcat(dir,base,strcat(int2str(N/1000),'k','z'),'.mat'),'z');
%save(strcat(dir,base,strcat(int2str(N/1000),'k','m'),'.mat'),'m');

%5. Gaussian dist in 1/8 sphere
base='sphereseg';
z = randn(N,3);
ind=z(:,1)>0;
z1=z(ind,1);
z2=z(ind,2);
z3=z(ind,3);
z=cat(2,z1,z2,z3);
N=size(z1,1);
figure,scatter3(z(:,1),z(:,2),z(:,3),'g')
m = rand(N,1);

xyzm= [z m]; 
mkdir(strcat(dir,base,int2str(N),'/'));
partition_mesh(xyzm,p,N,strcat(dir,base,int2str(N),'/'));
save(strcat(dir,base,int2str(N),'/z','.mat'),'z');
save(strcat(dir,base,int2str(N),'/m','.mat'),'m');

