% This script generates some simple point-and-mass distributions and saves
% them to Matlab binary file.

%N = 10000;
Nvec = logspace(4,5,2);
max_p = 4;
scenarios_dir = './experiments/paramsweep/scenarios/';

mkdir(scenarios_dir);

scenario_funcs = {@uniform, @gaussian, @point, @shell, @pole_alpha1_, @pole_alpha2_, @pole_alpha3_, @line_aligned, @line_unaligned, @spiral};

for i = 1:length(scenario_funcs)
    name = func2str(scenario_funcs{i});
    
    for N = Nvec
        % Ensure that the same random points are generated every time.
        seed = 131345;
        rng(seed, 'twister');
        xyzm = scenario_funcs{i}(N);

        scenario_dir = strcat(scenarios_dir,name,int2str(N),'/');
        mkdir(scenario_dir);
        for p = 1:max_p
            partition_mesh(xyzm,p,N,scenario_dir);
        end
        
        figure;
        title(name);
        scatter3(xyzm(:,1),xyzm(:,2),xyzm(:,3));
        
    end
end

function xyzm = uniform(N)
    xyzm = rand(N,4);
end
function xyzm = gaussian(N)
    xyz = randn(N,3);
    m = rand(N,1);
    xyzm = [xyz m];
end
function xyzm = pole_alpha1_(N)
    xyzm = pole(N,1);
end
function xyzm = pole_alpha2_(N)
    xyzm = pole(N,2);
end
function xyzm = pole_alpha3_(N)
    xyzm = pole(N,3);
end
function xyzm = pole(N, alpha)
    xyzm = shell(N);
    for i = 1:N
        r = rand();
        xyzm(i,1:3) = xyzm(i,1:3) .* (r^alpha);
    end
end
function xyzm = point(N) 
    xyz = zeros(N,3);
    m = rand(N,1);
    xyzm = [xyz m];
end
function xyzm = shell(N)
    R=1;
    s = 2*pi*rand(N,1);
    t = 2*pi*rand(N,1);
    x = R*cos(s).*sin(t);
    y = R*sin(s).*sin(t);
    z = -R*cos(t);
    m = rand(N,1);
    xyzm = [x y z m];
end
function xyzm = line_aligned(N)
    x = rand(N,1)*2 - 1;
    yz = zeros(N,2);
    m = rand(N,1);
    xyzm = [x yz m];
end
function xyzm = line_unaligned(N)
    x = rand(N,1)*2 - 1;
    m = rand(N,1);
    xyzm = [x x x m];
end
function xyzm = spiral(N)
    num_loops = 3;
    x = rand(N,1)*2 - 1;
    y = cos(x*pi*num_loops);
    z = sin(x*pi*num_loops);
    m = rand(N,1);
    xyzm = [x y z m];
end