#!/bin/bash

fmm3d_path=./fmm3d
paramsweep_dir=./experiments/paramsweep
result_dir=${paramsweep_dir}/results
scenario_dir=${paramsweep_dir}/scenarios

# default params
eta=0.5
Nlevels=3
theta=0.37

num_iter=4
meshonly=false

scenario_name=$1
param=$2
param_space=$3
min=$4
max=$5
step_inc=$5
num_steps=$6
num_process=$7


if [ $param_space == "step" ]; then
max=$(($min + $step_inc * $((num_steps-1))))
fi

#run_id="${scenario_name}-${param}=${param_space}(min=${min},max=${max},nsteps=${num_steps})-nprocs=${num_process}"
run_id="${scenario_name}_param${param}_${param_space}_${min}_${max}_${num_steps}_p${num_process}"
run_dir=${result_dir}/${run_id}

#if [ -d "$run_dir" ]; then
#echo "Directory for run-id \"$run_id\" already exists. Deleting and rewriting."
#rm -r "$run_dir"
#else
echo "Starting run with run-id \"$run_id\"."
#fi

log_dir=${run_dir}/logs
out_dir=${run_dir}/out

mkdir -p "$run_dir"
mkdir -p "$log_dir"
mkdir -p "$out_dir"

mfile="${scenario_dir}/${scenario_name}/m.mat"
zfile="${scenario_dir}/${scenario_name}/z.mat"

mpi_opts="-n $num_process"
for (( j=0; j < $num_iter; j++ )) do
echo "Working on run-id \"$run_id\", iteration ${j}/$(($num_iter-1))."
for (( i=0; i < $num_steps; i++ )) do
if [ $param_space == "lin" ]; then
value=$(bc -l <<< "$min + ($i/$((num_steps-1))) * ($max-$min)")
elif [ $param_space == "step" ]; then
value=$(($min + $step_inc * i))
elif  [ $param_space == "log" ]; then
value=$(python -c "print(int($min**(1 + ($max-1) * (float($i)/$((num_steps-1)))))")
else
echo "Invalid parameter space \"$param_space\"."
continue
fi

if  [ $param == "T" ]; then
theta=$value
elif [ $param == "n" ]; then
eta=$value
elif [ $param == "l" ]; then
Nlevels=$value
else
echo "Invalid parameter \"$param\"."
continue
fi

# mesh output:
if $meshonly; then
outfile="${out_dir}/mesh${i}.mat"
fmm3d_opts="-o -1 -T $theta -n $eta -l $Nlevels -z $zfile -m $mfile -f $outfile"
logpath="${log_dir}/mesh${i}_iter${j}.log"
else
outfile="${out_dir}/step${i}.dat"
fmm3d_opts="-T $theta -n $eta -l $Nlevels -z $zfile -m $mfile -f $outfile"
logpath="${log_dir}/step${i}_iter${j}.log"
fi
mpi_args="$mpi_opts $fmm3d_path $fmm3d_opts"
mpirun $mpi_args > $logpath
done
done

