#!/usr/bin/env python

import sys
import os
import argparse
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.cm as cm
import itertools
import re

def weakscaling(scenarios, param_name, param_short,
                param_space, param_min, param_max, num_steps,
                num_process, iterations=4):
  param_vec = np.linspace(param_min,param_max,num=num_steps)
  print(param_vec)
  '''Warning: only works for nprocs=1'''
  #program_steps = ['Alloc','Sort','Step #1','Step #2',
  #         'Step #3H','Step #3','Eval','TOTAL']
  program_steps = ['Alloc','Tree','P2M','M2M','M2L','L2L','P2P','Total']
    
  all_scenarios_times_min = []
  all_scenarios_times_max = []
  all_scenarios_times = []
  labels = []
  savedir = './experiments/paramsweep/results/'
  for scenario in scenarios:
    logdir = './experiments/paramsweep/results/{0}_param{1}_{2}_{3}_{4}_{5}_p{6}/logs'.format(scenario,param_short,param_space,param_min,param_max,num_steps,num_process)
    #logdir = './experiments/paramsweep/results/{0}-{1}={2}(min={3},max={4},nsteps={5})-nprocs={6}/logs'.format(scenario,param_short,param_space,param_min,param_max,num_steps,num_process)
    print('Working on log directory: {0}'.format(logdir))

    #HACK: remove numbers from scenario name for cleaner plot labels
    #leadingdigits = re.compile(r'^\d+')
    #scen = ["".join(x) for _, x in itertools.groupby(scenario, key=str.isdigit)]
    scen = scenario.split('100000')[0]

    avg_time = [0]*num_steps
    min_time = [sys.float_info.max]*num_steps
    max_time = [0]*num_steps
    scenario_max_time = 0
    for iter in range(iterations):
      time = np.zeros(num_steps)
      for step in range(num_steps):
        name = logdir+'/step'+str(step)+'_iter'+str(iter)+'.log'
        try:
		      log1 = open(name,'r')
        except:
          print('invalid file name:', name)
          # HACK: skip over step e.g. when comparing cases with different nos of steps
          time[step] = np.nan
          continue
          #sys.exit()
        t = np.zeros((8))
        for line in log1:
		      try:
		        r = int(line.split(', ')[0].split('rank ')[1])
		        program_step = line.split(': ')[0].split(', ')[1]
		        s = program_steps.index(program_step)
		        t[s] = float(line.split(': ')[1])
		      except:
		        continue
        
        # ignore total in log files, use sum of 7 subroutines instead
        total = sum(t[:-1])
        time[step] = total
                
      max_time = [x if x > y else y for x, y in zip(max_time, time)]
      min_time = [x if x < y else y for x, y in zip(min_time, time)]
      avg_time = [x + y/iterations for x, y in zip(avg_time, time)]
    # normalise times and add to vecs
    scenario_max_time = max(avg_time)
    all_scenarios_times_max += [max_time/scenario_max_time]
    all_scenarios_times_min += [min_time/scenario_max_time]
    all_scenarios_times += [avg_time/scenario_max_time]
    labels += [scen]
    print(scenario, 'time:', time)
    print(scenario, 'max avg time:', scenario_max_time)

  # find minimum average time
  print('scenario %s, param %s, minimum time %.4f, optimal param %.4f' % (scenario,param_name,min(avg_time),param_vec[avg_time.index(min(avg_time))]))

  #colors = ['r','b','g','c','m','y','k','w']
  colors = list(reversed(cm.rainbow(np.linspace(0, 1, len(all_scenarios_times)))))
  symbols = ['-o','-^','-s','-+']
  
  plt.figure()
  for t, min_t, max_t, s, c in zip(all_scenarios_times, all_scenarios_times_min, all_scenarios_times_max, symbols, colors):
    plt.errorbar(param_vec, t, yerr=[[y-x for x, y in zip(min_t, t)],[x - y for x, y in zip(max_t, t)]], fmt=s, color=c, capsize=6, capthick=2, elinewidth=2)
  # plt.title('CPU time on ' + str(num_process) + ' process.',fontsize=fsize)
  if param_name == 'Theta':
    plt.xlabel(r'$\theta$',fontsize=fsize)
  elif param_name == 'Eta':
    plt.xlabel(r'$\eta$',fontsize=fsize)
  elif param_name == 'Nlevels':
    plt.xlabel(r'$L$',fontsize=fsize)
  else:
    plt.xlabel(param_name,fontsize=fsize)
  plt.ylabel('Normalised CPU Time (s)',fontsize=fsize)
  plt.legend(labels,loc='best',fontsize=fsize2)
  x1,x2,y1,y2 = plt.axis()
  #plt.axis((x1,x2*1.05,0,y2))
  plt.axis((min(param_vec),max(param_vec)*1.05,0,y2))
  if param_name == 'NLevels':
    plt.xticks(param_vec, fontsize=fsize2)
  else:
    plt.xticks(fontsize=fsize2)
  plt.yticks(fontsize=fsize2)
  plt.savefig(savedir+'/plot_'+scenario+'_'+param_name+'.png')
  plt.show()
  return

def valid_scenario(string):
  return string

def valid_param(string):
  if string not in ['l', 'T', 'n']:
    msg = "%r is not a valid parameter" % string
    raise argparse.ArgumentTypeError(msg)
  return string
    

def main():
  global fsize,fsize2
  fsize=20
  fsize2=16
  parser = argparse.ArgumentParser(
      description='Plot time taken for different values of an algorithm parameter.')
  parser.add_argument('param', choices=['n', 'l', 'T'],
                      help='the parameter to create a plot for')
  parser.add_argument('nprocs', type=int, default=1,
                      help='number of processes in the runs to create a plot for')
  parser.add_argument('scenarios', metavar='scenario', type=valid_scenario, nargs='+',
                      help='a scenario to include in the plot')
  args = parser.parse_args()
  print(args)
    
  nprocs = args.nprocs
  scenarios = args.scenarios
  if (args.param == 'l'):
    weakscaling(scenarios, 'NLevels', 'l', 'step', 1, 5, 5, nprocs)
  if (args.param == 'T'):
    weakscaling(scenarios, 'Theta',   'T', 'lin',  0, 0.8, 16, nprocs)
  if (args.param == 'n'):
    weakscaling(scenarios, 'Eta',     'n', 'lin',  0, 1, 16, nprocs)
    
if __name__ == "__main__":
  sys.exit(main())
 
