/* cudaeval.h -- CUDA host code for direct evaluation. */

/* S. Engblom and A. Goude 2011-10-19 */

/*#ifdef __cplusplus

#endif*/

//#ifdef CUDASUPPORT
#include "fmm.h"
#include "cuda.h"
#include "driver_types.h"

inline int imin(int x,int y)
{
  return x < y ? x : y;
}
// struct collecting all data to send to the GPU
struct cudavariables {
  int Nf;
  double *ex,*ey,*ez,*zx,*zy,*zz,*mm,*qr,*pr;
  int *ixptr,*jxptr;
  cudaEvent_t start,stop;
  cudaEvent_t mpstart,mpstop;
  cudaEvent_t initstart,initstop;
  cudaEvent_t stepm2mstart,stepm2mstop;
  cudaEvent_t stepp2pstart,stepp2pstop;
  cudaEvent_t stepm2psstart,stepm2psstop;
  cudaEvent_t fulltimestart,fulltimestop;
  cudaEvent_t reorder1start,reorder1stop;
  cudaEvent_t reorder2start,reorder2stop;
  cudaEvent_t sortstart,sortstop;
  cudaEvent_t checkstart,checkstop;
  //SORT_REAL *z0;
  //SORT_REAL *d0;
  int *ix,*jx;
  // allocated on host memory, but internal variables on GPU memory
  //mpSparse* connect;
  //SORT_REAL *dabs;

#ifdef __cplusplus

   cudaDeviceProp info;

#else // !_cplusplus

   struct cudaDeviceProp info;

#endif
  int evalonly; //set if all evaluation is to be performed on eval points
  double *eta;
};

//Keeps track of all memory allocations making it easy to spot memory leaks
//Make all allocations through these functions
cudaError_t cudaMallocDebug(void **addr,size_t size);
cudaError_t cudaFreeDebug(void *addr);
void printcudaalloccount(char *message);
int getcudaalloccount();
void resetalloccount();

//Checks the error message of a cuda call and aborts the code if a cuda call fails
#define cudasafe(command,message) cudasafe2(command,message,#command,__FILE__,__LINE__)
void cudasafe2(cudaError_t error,const char *message,const char* command,const char *file,int line);
void cudasafeMalloc(void **addr,size_t size);
void checkcudaerror(const char *message);
void cudareset();
void cudastart();

//#ifdef CHECKCUDACALLS
#define CHECKCUDAERROR cudasafe(cudaThreadSynchronize(),"check");
void checkcudaerrorwrapper(int line);
#define CHECKCUDAERROR2 checkcudaerrorwrapper(__LINE__);
//#else
//#define CHECKCUDAERROR
//#define CHECKCUDAERROR2
//#endif

// CudaMemcpy and CudaMemset with automatic check of the return value. Aborts if call fails
void cudaSafeMemcpy(void *dst,const void *src,size_t count,
                    enum cudaMemcpyKind kind,const char *str);

void cudaSafeMemset(void *dst,int value,size_t count,const char* str);

cudaError_t cudaGetLastErrorDummy();

void direct_eval_cuda(int N,
		      const double *zx,const double *zy,const double *zz,
		      const double *mm,int NE,
		      const double *ex,const double *ey, const double *ez,
		      double *pr,
		      double *qr,
		      int pot,enum SMOOTHER smooth);


/* The function prototypes below can be called from the intermediate direct.cpp
 * Cuda code can then be written in cudaeval.cu as with the direct function above.
 */
void mpexp_direct_interact_cuda2(double *pr,
			   const double *zx,const double *zy,const double *zz,
			   const double *mm,
			   const int *ix,
			   int begin1,int end1,int begin2,int end2,
			   int pot,enum SMOOTHER smooth);


void mpexp_direct_interact_cuda(double *qr,
			  const double *ex,const double *ey,const double *ez,
			  const int *jx,int begin1,int end1,
			  const double *zx,const double *zy,const double *zz,
			  const double *mm,
			  const int *ix,int begin2,int end2,
			  int pot,enum SMOOTHER smooth);

//#endif /* CUDASUPPORT */
