/* fmmshift.h */

/* S. Engblom 2009-10-08 (Revision) */
/* S. Engblom 2009-04-15 (Major revision) */
/* S. Engblom 2007-05-29 */

#ifndef __fmmshift_h
#define __fmmshift_h

#include "fmm.h"

// Cartesian to spherical coordinate transformation
void cart2spher(const double dx,const double dy,const double dz,double *sph);
// allocate workspace (temporary) arrays and set parameters
void shift_alloc(int p,int pot,int maxm2l);
// local and multipole harmonic expansions
void evalMultipole(double rho, double alpha, double beta, dcmplx **Ynm);
void evalMultipoleZ(double rho, double *Ynm);
void evalLocal(double rho, double alpha, double beta, dcmplx **Ynm);
void evalLocalZ(double rho, double *Ynm);
/* Definition of the three shift-operators used by the
   FMM. Performance is critical in general and in particular for the
   m2l-shift. */
void shift_m2m(struct mpexp *this1, const struct mpexp *that1);
void shift_m2mZ(struct mpexp *this1, const struct mpexp *that1);
void shift_m2l(struct mpexp *this1, struct mpexp *that1,
               const int *ix,int begin,int end, int symm);
void shift_m2lZ(struct mpexp *this1, struct mpexp *that1,
               const int *ix,int begin,int end, int symm);
void shift_m2lZsimple(int rank,double *sph,
               const dcmplx *thism,dcmplx *thisl,
               const dcmplx *thatm,dcmplx *thatl);
void shift_l2l(struct mpexp *this1, const struct mpexp *that1);
void shift_l2lZ(struct mpexp *this1, const struct mpexp *that1);
// alternate versions (TODO: which is faster?)
void shift_m2msimple(double *sph,dcmplx *thism,const dcmplx *thatm);
void shift_m2lsimple(int rank,double *sph,
               const dcmplx *thism,dcmplx *thisl,
               const dcmplx *thatm,dcmplx *thatl);
void shift_l2lsimple(double *sph,dcmplx *thisl,const dcmplx *thatl);

void rotate_Multipole_scaled(dcmplx *thism, const dcmplx *thatm, const double* Wigner, const dcmplx beta_rot);
void inv_rotate_Multipole_scaled(dcmplx *thism, const dcmplx *thatm, const double* Wigner, const dcmplx beta_rot);
void rotate_Local_scaled_permuted(dcmplx *thism, const dcmplx *thatm, const double* Wigner, const dcmplx beta_rot);
void inv_rotate_Local_scaled_permuted(dcmplx *thism, const dcmplx *thatm, const double* Wigner, const dcmplx beta_rot);
void buildWigner_d_scaled(double alpha, double *Wigner_d);

#endif /* __fmmshift_h */
