function h = meshconnectplot3(M,C,S,index,N,full)
%MESHCONNECTPLOT3 Plot FMM mesh with connectivity information in 3D.
%   MESHCONNECTPLOT3(M,C,S,index) plots the FMM mesh M in 3D as obtained via the 'out'
%   property in FMM3D.
% N is the number of boxes per process (block size in C).
% index is the global index of a box (range 1:N*nprocs).
% full is a flag for whether C is lower-triangular or full-rank.

% J. Bull 2015-10-26

%example of how to generate the input arrays:
%[M,C,S] = fmm3d(z,m,optional_args);

%plot sparsity with colours

%option to make full from lower triangular
if(~full),
    C = C''+C';
end
figure,spy(C==-1,'g'); % far
hold;
spy(C==1,'r'); % near (do this second to display better)
%spy(eye(size(C,1)),'r');
hold off;

% ignore empty boxes at finest level
%j = find(sum(S));
%P = M(:,j);
%D = C(:,j);
% beware: entries of D do not match indices

c = [0.5 0.5 0.5];
figure, meshplot3(M,'color',c);
hold;
axis equal;
cameramenu;
xlabel('x');
ylabel('y');
zlabel('z');

i = index;
% find which proc i belongs to
proc = floor((i-1)/N)

% box i
MM = M(:,i);
fprintf('source box: %d\n',MM);
c = [1 0 0]; % red
boxsurfaceplot(MM,c);

% near interaction of box i
%make C full first
if(~full),
    C = C''+C';
end
j = find(C(:,i) > 0);
myj = j(j > proc*N & j <= (proc+1)*N);
MM = M(:,myj);
%now remove zero boxes
P=MM(:,sum((MM==0))==0);
c = [0 0 1]; % blue
boxsurfaceplot(P,c);

% halo near interactions
j_halo = j(j <= proc*N);
j_halo = [j_halo; j(j > (proc+1)*N)];
MM = M(:,j_halo);
%now remove zero boxes
P=MM(:,sum((MM==0))==0);
c = [0 1 1]; % cyan
boxsurfaceplot(P,c);

% cluster-to-cluster (far) interaction of box i
j = find(C(:,i) < 0);
myj = j(j > proc*N & j <= (proc+1)*N);
MM = M(:,myj);
%now remove zero boxes
P=MM(:,sum((MM==0))==0);
c = [0 1 0]; % green
boxsurfaceplot(P,c);
%hold off;

% halo far interactions
j_halo = j(j <= proc*N);
j_halo = [j_halo; j(j > (proc+1)*N)];
MM = M(:,j_halo);
%now remove zero boxes
P=MM(:,sum((MM==0))==0);
c = [1 1 0]; % yellow
boxsurfaceplot(P,c);

% here, optimized cross-level cluster-to-cluster (far) interactions are
% displayed by plotting the receiving parent (a visual approximation)
%j2 = find(C(:,i) == -2);
%j2 = floor((j2-1)/4)+1;
%MM = M(:,j2);
%now remove zero boxes
%P=MM(:,sum((MM==0))==0);
%c = [0 0 1]; % blue
%boxsurfaceplot(MM,c);

% in this version all receiving boxes are plotted, this is in some
% sense more correct but does not display the true complexity
%j2 = find(C(:,i) == -2);
%j2a = floor((j2(j2 > i)-1)/4)+1;
%MM = M{l-1}(:,j2a);

%j2b = j2(j2 < i);
%MM = M(:,j2b);

%axis square, axis([0 1 0 1]);
%set(gca,'xtick',[],'ytick',[]);
%set(gcf,'PaperPositionMode','auto');
%set(gcf,'Position',[100 100 250 250]);
print('mesh.eps','-depsc');
