function h = meshplot3(M,varargin)
%MESHPLOT3 Plot FMM mesh in 3D.
%   MESHPLOT3(M) plots the FMM mesh M in 3D as obtained via the 'out'
%   property in FMM3D.
%   H = MESPLOT3(M,...) sends additional arguments to PLOT and
%   returns a handle to the resulting plot.
%
%   Example:
% N = 100;
% xx = rand(N,1);
% yy = rand(N,1);
% zz = rand(N,1);
% z = cat(2,xx,yy,zz);
% m = rand(N,1);
% [M,C] = fmm3d(z,m,'tol',1e-7,'ndirect',5,'out','mesh');
% l = 1;
% figure, meshplot3(M{l},'k');
% hold;
% plot3(z(:,1),z(:,2),z(:,3),'bo');
% axis([-4 4 -4 4 0 1e3]);
%
%   See also FMM3D, MESHPLOT, PLOT3.

% J. Bull 2015-10-26
%endpt=size(M(1,:),2)/2;
endpt=size(M(1,:),2);
% 1st Hilbert curve:
x1 = M([1 1 1 1 1 1 1 1 1],1:endpt)+[-M(4,1:endpt); -M(4,1:endpt); -M(4,1:endpt); -M(4,1:endpt); M(4,1:endpt); M(4,1:endpt); M(4,1:endpt); M(4,1:endpt); -M(4,1:endpt)];
y1 = M([2 2 2 2 2 2 2 2 2],1:endpt)+[-M(5,1:endpt); -M(5,1:endpt); M(5,1:endpt); M(5,1:endpt); M(5,1:endpt); M(5,1:endpt); -M(5,1:endpt); -M(5,1:endpt); -M(5,1:endpt)];
z1 = M([3 3 3 3 3 3 3 3 3],1:endpt)+[-M(6,1:endpt); M(6,1:endpt); M(6,1:endpt); -M(6,1:endpt); -M(6,1:endpt); M(6,1:endpt); M(6,1:endpt); -M(6,1:endpt); -M(6,1:endpt)];

% 2nd Hilbert curve:
x2 = M([1 1 1 1 1 1 1 1 1],1:endpt)+[-M(4,1:endpt); -M(4,1:endpt); M(4,1:endpt); M(4,1:endpt); M(4,1:endpt); M(4,1:endpt); -M(4,1:endpt); -M(4,1:endpt); -M(4,1:endpt)];
y2 = M([2 2 2 2 2 2 2 2 2],1:endpt)+[-M(5,1:endpt); -M(5,1:endpt); -M(5,1:endpt); -M(5,1:endpt); M(5,1:endpt); M(5,1:endpt); M(5,1:endpt); M(5,1:endpt); -M(5,1:endpt)];
z2 = M([3 3 3 3 3 3 3 3 3],1:endpt)+[-M(6,1:endpt); M(6,1:endpt); M(6,1:endpt); -M(6,1:endpt); -M(6,1:endpt); M(6,1:endpt); M(6,1:endpt); -M(6,1:endpt); -M(6,1:endpt)];

if nargout > 0
  h = plot3([x1;x2],[y1;y2],[z1;z2],varargin{:});
else
  plot3([x1;x2],[y1;y2],[z1;z2],varargin{:});
end
