function [MM,CC] = parse_mesh(l,p,i,base,plot)

% Parallel mesh+connectivity parsing and optional plotting. Arguments are:
%l: the level of the FMM tree to plot;
%p: number of procs that were used for run;
%i: the global number of the box to use as the source box in meshconnectplot3;
%base: the basename for some mesh files, e.g. 'examples/demisphere/mesh1000';
%plot: flag to plot mesh and connectivity.

%Output is:
%M: a matrix of all boxes in the domain at level l;
%C: a matrix of connections for the whole domain at level l.
% both are in full (not sparse) format.

%full: 0 or 1. whether arrays are lower-triangular (0) or full-rank (1);
full = 0;

%load mesh files
if(p==1),
    mesh.('p0')=load(strcat(base,'.mat'));
else,
    for j=0:p-1,
        mesh.(strcat('p',int2str(j)))=load(strcat(base,'_',int2str(j),'.mat'));
    end
end

%get arrays from meshes
N=8^l;
Ml=strcat('M',int2str(l));
Cl=strcat('C',int2str(l));

for j=0:p-1,
  var=strcat('p',int2str(j));
  M.(strcat('p',int2str(j)))=mesh.(var).(Ml);
  C.(strcat('p',int2str(j)))=mesh.(var).(Cl);
end

if(p>1),
  Hl=strcat('H',int2str(l));
  for j=0:p-1,
    var=strcat('p',int2str(j));
    H.(strcat('p',int2str(j)))=mesh.(var).(Hl);
  end
end

%reshape mesh, connectivity
for j=0:p-1,
  var=strcat('p',int2str(j));
  M.(var)=reshape(M.(var),6,N);
  C.(var)=reshape(C.(var),N,N);
end

%combine connectivity and halos
CC=zeros(p*N,p*N);
if(full==1),
  % loop over procs
  for j=0:p-1,
    var=strcat('p',int2str(j));
    C.(var)=C.(var)''+C.(var)';
    CC(j*N+1:(j+1)*N,j*N+1:(j+1)*N)=C.(var);
    %lower triangle
    for k=0:j-1,
      CC(k*N+1:(k+1)*N,j*N+1:(j+1)*N)=H.(var){k+1};
    end
    %upper triangle
    for k=j+1:p-1,
      CC(k*N+1:(k+1)*N,j*N+1:(j+1)*N)=H.(var){k};
    end
  end
else,
  for j=0:p-1,
    var=strcat('p',int2str(j));
    CC(j*N+1:(j+1)*N,j*N+1:(j+1)*N)=C.(var);
    %lower triangle
    for k=0:j-1,
      CC(k*N+1:(k+1)*N,j*N+1:(j+1)*N)=H.(var){k+1};
    end
  end
end

%check symmetry
if(full==1),
    Cfl=single(CC);
    disp('Is C symmetric?');
    issymmetric(Cfl)
end

%combine mesh sub-arrays
MM=[];
for j=0:p-1,
  var=strcat('p',int2str(j));
  MM=[MM M.(var)];
end

% TODO: generalise to any number of procs
if(plot),
  %plot partitioned mesh in fancy colours
  figure, meshplot3(M.p0,'color','r');
  hold;
  if(p>1),
    meshplot3(M.p1,'color','b');
  end
  if(p>2),
    meshplot3(M.p2,'color','g');
  end
  if(p>3),
    meshplot3(M.p3,'color','c');
  end
  if(p>4),
    meshplot3(M.p4,'color','m');
  end
  if(p>5),
    meshplot3(M.p5,'color','y');
  end
  if(p>6),
    meshplot3(M.p6,'color','k');
  end
  if(p>7),
    meshplot3(M.p7,'color','r');
  end

  axis equal;
  cameramenu;
  xlabel('x');
  ylabel('y');
  zlabel('z');
  hold off;

  %plot connections and halos
  meshconnectplot3(MM,CC,CC,i,N,full);
end

